@@ -1,9 +1,34 @@
 import certifi
 import ldap
-from ldap.modlist import addModlist, modifyModlist
+from ldap.modlist import addModlist
 from flask import current_app, _app_ctx_stack
 
 
+def modify_modlist(oldAttrs, newAttrs):
+    modifications = []
+    oldKeySet = set(oldAttrs.keys())
+    newKeySet = set(newAttrs.keys())
+    for key in oldKeySet - newKeySet:
+        modifications.append((ldap.MOD_DELETE, key, None))
+    for key, newValue in newAttrs.items():
+        oldValue = oldAttrs.get(key, [])
+        if not newValue:
+            modifications.append((ldap.MOD_DELETE, key, None))
+        else:
+            oldValueSet = set(oldValue)
+            newValueSet = set(newValue)
+            addList = list(newValueSet - oldValueSet)
+            removeList = list(oldValueSet - newValueSet)
+            if addList and removeList:
+                modifications.append((ldap.MOD_REPLACE, key, newValue))
+            elif addList:
+                modifications.append((ldap.MOD_ADD, key, addList))
+            elif removeList:
+                modifications.append((ldap.MOD_DELETE, key, removeList))
+
+    return modifications
+
+
 class LDAP(object):
 
     def __init__(self, app=None):
@@ -22,7 +47,11 @@ class LDAP(object):
         conn = ldap.initialize(uri)
         if (uri.startswith('ldaps:')):
             conn.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,ldap.OPT_X_TLS_DEMAND)
-            conn.set_option(ldap.OPT_X_TLS_CACERTFILE, certifi.where())
+            # OPT_X_TLS_CACERTFILE does not work on OS X for some reason but validation seem to work
+            try:
+                conn.set_option(ldap.OPT_X_TLS_CACERTFILE, certifi.where())
+            except ValueError:
+                pass
             conn.set_option(ldap.OPT_X_TLS_NEWCTX,0)
         conn.simple_bind_s(current_app.config['LDAP_BINDDN'], current_app.config['LDAP_SECRET'])
         return conn
@@ -206,7 +235,7 @@ class Entry(object, metaclass=ModelBase):
             self.new = False
         else:
             new_attributes = self.prep_attr_dict_for_ldap(self._attributes)
-            mod_list = modifyModlist(self._initial_attributes, new_attributes)
+            mod_list = modify_modlist(self._initial_attributes, new_attributes)
             current_app.extensions['ldap'].connection.modify_s(self.dn, mod_list)
             self._initial_attributes = new_attributes
         return True
